from com.arm.debug.dtsl.configurations import DTSLv1
from com.arm.debug.dtsl.components import ConnectableDevice
from com.arm.debug.dtsl.components import DeviceInfo
from com.arm.debug.dtsl.components import PVCacheDevice
from com.arm.debug.dtsl.components import PVCacheMemoryAccessor
from com.arm.debug.dtsl.components import PVCacheMemoryCapabilities
from com.arm.debug.dtsl.components import ModelInstTraceSource
from com.arm.debug.dtsl.components import ModelStreamingTraceCapture
from com.arm.debug.dtsl.components import ModelLiveStoredStreamingTraceCapture
from com.arm.debug.dtsl.components import StreamingTraceMultipleSinkConnectionWrapper
from com.arm.debug.dtsl.components import CadiSyncSMPDevice

coreNames = ["ARM_Cortex-A32_0", "ARM_Cortex-A32_1", "ARM_Cortex-A32_2", "ARM_Cortex-A32_3", "ARM_Cortex-M0+", "ARM_Cortex-M3"]
traceDeviceNames = ["host.cluster0.cpu0.modelInstTrace",
                    "host.cluster0.cpu1.modelInstTrace",
                    "host.cluster0.cpu2.modelInstTrace",
                    "host.cluster0.cpu3.modelInstTrace",
                    "se.cpu.modelInstTrace",
                    "extsys0.cpu.modelInstTrace"]
CONTENTS, TAGS = 0, 1


class DtslScript(DTSLv1):
    @staticmethod
    def getOptionList():
        return [
            DTSLv1.tabSet("options", "Options", childOptions=[
                DtslScript.getOptionTraceBufferTabPage(),
                DtslScript.getCoreTraceTabPage(),
            ])
        ]
    
    def __init__(self, root):
        DTSLv1.__init__(self, root)
        
        '''Do not add directly to this list - first check if the item you are adding is already present'''
        self.mgdPlatformDevs = []
        
        # Locate devices on the platform and create corresponding objects
        self.discoverDevices()
        
        self.exposeCores()
        
        self.setupCadiSyncSMP()
        
    
    # +----------------------------+
    # | Target dependent functions |
    # +----------------------------+
    
    def discoverDevices(self):
        '''Find and create devices'''
        
        self.cores = {}
        self.createModelCore("host.cluster0.cpu0", "ARM_Cortex-A32_0", "Cortex-A32")
        self.createModelCore("host.cluster0.cpu1", "ARM_Cortex-A32_1", "Cortex-A32")
        self.createModelCore("host.cluster0.cpu2", "ARM_Cortex-A32_2", "Cortex-A32")
        self.createModelCore("host.cluster0.cpu3", "ARM_Cortex-A32_3", "Cortex-A32")
        self.createModelCore("se.cpu", "ARM_Cortex-M0+", "Cortex-M0+")
        self.createModelCore("extsys0.cpu", "ARM_Cortex-M3", "Cortex-M3")
        
        self.cluster0cores = []
        self.addModelCoreToCluster(self.cluster0cores, "host.cluster0.cpu0")
        self.addModelCoreToCluster(self.cluster0cores, "host.cluster0.cpu1")
        self.addModelCoreToCluster(self.cluster0cores, "host.cluster0.cpu2")
        self.addModelCoreToCluster(self.cluster0cores, "host.cluster0.cpu3")
        
        self.caches = {}
        self.createModelCache("host.cluster0.cpu0.l1icache", "l1icache_0")
        self.createModelCache("host.cluster0.cpu0.l1dcache", "l1dcache_0")
        self.createModelCache("host.cluster0.cpu1.l1icache", "l1icache_1")
        self.createModelCache("host.cluster0.cpu1.l1dcache", "l1dcache_1")
        self.createModelCache("host.cluster0.cpu2.l1icache", "l1icache_2")
        self.createModelCache("host.cluster0.cpu2.l1dcache", "l1dcache_2")
        self.createModelCache("host.cluster0.cpu3.l1icache", "l1icache_3")
        self.createModelCache("host.cluster0.cpu3.l1dcache", "l1dcache_3")
        self.createModelCache("host.cluster0.l2_cache", "l2_cache_0")
        self.addPVCache("host.cluster0.cpu0", "host.cluster0.cpu0.l1icache", "host.cluster0.cpu0.l1dcache", "host.cluster0.l2_cache")
        self.addPVCache("host.cluster0.cpu1", "host.cluster0.cpu1.l1icache", "host.cluster0.cpu1.l1dcache", "host.cluster0.l2_cache")
        self.addPVCache("host.cluster0.cpu2", "host.cluster0.cpu2.l1icache", "host.cluster0.cpu2.l1dcache", "host.cluster0.l2_cache")
        self.addPVCache("host.cluster0.cpu3", "host.cluster0.cpu3.l1icache", "host.cluster0.cpu3.l1dcache", "host.cluster0.l2_cache")
        # DSCORE-9063
        # self.addPVCache("se.cpu", "null", "null")
        # self.addPVCache("extsys0.cpu", "null", "null")
        
    def createModelCore(self, deviceName, dtslName, coreDefinition):
        try:
            self.cores[deviceName] = ConnectableDevice(self, self.findDevice(deviceName), dtslName)
            deviceInfo = DeviceInfo("core", coreDefinition)
            self.cores[deviceName].setDeviceInfo(deviceInfo)
        except:
            # Core failed to be added
            pass
    
    def exposeCores(self):
        '''Expose cores'''
        for device in self.cores.values():
            self.addDeviceInterface(device)
    
    def addModelCoreToCluster(self, clusterGroup, deviceName):
        if deviceName in self.cores.keys():
            clusterGroup.append(self.cores[deviceName])
    
    def createModelCache(self, deviceName, dtslName):
        cacheDevice = self.findDevice(deviceName)
        pvCacheDevice = PVCacheDevice(self, cacheDevice, dtslName)
        self.caches[deviceName] = pvCacheDevice
        self.mgdPlatformDevs.append(pvCacheDevice)
    
    def addPVCache(self, devName, l1iName, l1dName, l2Name=None):
        '''Add cache devices'''
        
        # Only add the cache if the core has been found
        if devName in self.cores.keys():
            
            l1i = self.caches[l1iName]
            l1d = self.caches[l1dName]
            dev = self.cores[devName]
            if l2Name:
                l2 = self.caches[l2Name]
                rams = [
                    (l1i, 'L1I', CONTENTS), (l1d, 'L1D', CONTENTS),
                    (l1i, 'L1ITAG', TAGS), (l1d, 'L1DTAG', TAGS),
                    (l2, 'L2', CONTENTS), (l2, 'L2TAG', TAGS)
                ]
            else:
                rams = [
                    (l1i, 'L1I', CONTENTS), (l1d, 'L1D', CONTENTS),
                    (l1i, 'L1ITAG', TAGS), (l1d, 'L1DTAG', TAGS),
                ]
            ramCapabilities = PVCacheMemoryCapabilities()
            for cacheDev, name, id in rams:
                cacheAcc = PVCacheMemoryAccessor(cacheDev, name, id)
                dev.registerAddressFilter(cacheAcc)
                ramCapabilities.addRAM(cacheAcc)
            dev.addCapabilities(ramCapabilities)
    
    def getTraceCaptureForSource(self, traceSource):
        if traceSource in self.traceSources.values():
            return self.traceCaptures.values()[self.traceSources.values().index(traceSource)]
        return None
    
    def getManagedDevices(self):
        '''Get the required list of managed devices for this configuration'''
        deviceList = self.mgdPlatformDevs[:]
        deviceList.extend(self.traceSources.values())
        deviceList.extend(self.traceCaptures.values())
        return deviceList
    
    def setupCadiSyncSMP(self):
        '''Create SMP device using CADI synchronization'''
        
        # cluster0 SMP
        smp = CadiSyncSMPDevice(self, "ARM_Cortex-A32x4 SMP Cluster 0", self.cluster0cores)
        self.addDeviceInterface(smp)
        
    
    # +--------------------------------+
    # | Callback functions for options |
    # +--------------------------------+
    
    def optionValuesChanged(self):
        '''Callback to update the configuration state after options are changed'''
        if not self.isConnected():
            self.setInitialOptions()
        
        self.updateDynamicOptions()
        
    def setInitialOptions(self):
        '''Set the initial options'''
        
        traceMode = self.getOptionValue("options.trace.traceCapture")
        traceBufferSize = self.getOptionValue("options.trace.traceCapture.modelCapture.traceBufferSize")
        streamID = 1
        sinkNum = 0
        self.traceSources = {}
        self.traceCaptures = {}
        rddiStreamingTrace = self.getStreamingTrace()
        multiSinkConnWrapper = StreamingTraceMultipleSinkConnectionWrapper(rddiStreamingTrace)
        for i in range(len(self.cores)):
            enableTraceForCore = self.getOptionValue("options.coreTrace.core%d" % i) and traceMode == "modelStreamingCapture"
            if enableTraceForCore:
                self.traceSources[i] = ModelInstTraceSource(self, self.findDevice(traceDeviceNames[i]), streamID, "model_instr_trace_%d" % i)
                self.traceSources[i].setEnabled(True)
                streamingCapture = ModelStreamingTraceCapture(self, multiSinkConnWrapper, "model_streaming_%d" % i, "model_trace_sink_%d" % sinkNum)
                self.traceCaptures[i] = ModelLiveStoredStreamingTraceCapture(self, streamingCapture, "model_streaming_capture_%d" % i)
                self.addTraceCaptureInterface(self.traceCaptures[i])
                self.addStreamTraceCaptureInterface(self.traceCaptures[i])
                self.traceCaptures[i].addTraceSource(self.traceSources[i], coreNames[i])
                self.setModelTraceBufferSize(traceBufferSize, self.traceCaptures[i])
                sinkNum += 1
            streamID += 1
        
        self.setManagedDeviceList(self.getManagedDevices())
        
    def updateDynamicOptions(self):
        '''Update the dynamic options'''
        
    def setModelTraceBufferSize(self, traceBufferSize, traceCapture):
        '''Configuration option setter method for the buffer size'''
        captureSize = 64*1024*1024
        if (traceBufferSize == "64MB"):
            captureSize = 64*1024*1024
        elif (traceBufferSize == "128MB"):
            captureSize = 128*1024*1024
        elif (traceBufferSize == "256MB"):
            captureSize = 256*1024*1024
        elif (traceBufferSize == "512MB"):
            captureSize = 512*1024*1024
        elif (traceBufferSize == "1GB"):
            captureSize = 1*1024*1024*1024
        elif (traceBufferSize == "2GB"):
            captureSize = 2*1024*1024*1024
        elif (traceBufferSize == "4GB"):
            captureSize = 4*1024*1024*1024
        elif (traceBufferSize == "8GB"):
            captureSize = 8*1024*1024*1024
        elif (traceBufferSize == "16GB"):
            captureSize = 16*1024*1024*1024
        elif (traceBufferSize == "32GB"):
            captureSize = 32*1024*1024*1024
        elif (traceBufferSize == "64GB"):
            captureSize = 64*1024*1024*1024
        elif (traceBufferSize == "128GB"):
            captureSize = 128*1024*1024*1024
        else:
            raise ValueError("Unexpected trace buffer size: ", traceBufferSize)
        
        traceCapture.setMaxCaptureSize(captureSize)
    
    # +------------------------------+
    # | Target independent functions |
    # +------------------------------+
    
    @staticmethod
    def getOptionTraceBufferTabPage():
        return DTSLv1.tabPage("trace", "Trace Buffer", childOptions=[
                DTSLv1.radioEnumOption("traceCapture", "Trace capture method", defaultValue="none",
                    values = [("none", "None"), DtslScript.getModelTraceOptions()]),
        ])
    
    @staticmethod
    def getModelTraceOptions():
        return ("modelStreamingCapture", "Model Trace",
                DTSLv1.infoElement("modelCapture", "", "", childOptions=[
                    DTSLv1.enumOption("traceBufferSize", "Trace Buffer Size", defaultValue="4GB",
                        values = [("64MB", "64MB"), ("128MB", "128MB"), ("256MB", "256MB"),
                                  ("512MB", "512MB"), ("1GB", "1GB"), ("2GB", "2GB"),
                                  ("4GB", "4GB"), ("8GB", "8GB"), ("16GB", "16GB"),
                                  ("32GB", "32GB"), ("64GB", "64GB"), ("128GB", "128GB")], isDynamic=False)
                ])
        )
    
    @staticmethod
    def getCoreTraceTabPage():
        return DTSLv1.tabPage("coreTrace", "Core Trace", childOptions=[
            DTSLv1.booleanOption("core0", "Enable ARM_Cortex-A32_0 trace", defaultValue=False),
            DTSLv1.booleanOption("core1", "Enable ARM_Cortex-A32_1 trace", defaultValue=False),
            DTSLv1.booleanOption("core2", "Enable ARM_Cortex-A32_2 trace", defaultValue=False),
            DTSLv1.booleanOption("core3", "Enable ARM_Cortex-A32_3 trace", defaultValue=False),
            DTSLv1.booleanOption("core4", "Enable ARM_Cortex-M0+ trace", defaultValue=False),
            DTSLv1.booleanOption("core5", "Enable ARM_Cortex-M3 trace", defaultValue=False),
        ])
    
