from com.arm.debug.dtsl.configurations import DTSLv1
from com.arm.debug.dtsl.components import ConnectableDevice
from com.arm.debug.dtsl.components import DeviceInfo
from com.arm.debug.dtsl.components import ModelInstTraceSource
from com.arm.debug.dtsl.components import ModelStreamingTraceCapture
from com.arm.debug.dtsl.components import ModelLiveStoredStreamingTraceCapture

coreNames = ["ARM_Cortex-M55"]
traceDeviceNames = ["cpu0.modelInstTrace"]


class DtslScript(DTSLv1):
    @staticmethod
    def getOptionList():
        return [
            DTSLv1.tabSet("options", "Options", childOptions=[
                DtslScript.getOptionTraceBufferTabPage(),
                DtslScript.getCoreTraceTabPage(),
            ])
        ]
    
    def __init__(self, root):
        DTSLv1.__init__(self, root)
        
        '''Do not add directly to this list - first check if the item you are adding is already present'''
        self.mgdPlatformDevs = []
        
        # Locate devices on the platform and create corresponding objects
        self.discoverDevices()
        
        self.exposeCores()
        
    
    # +----------------------------+
    # | Target dependent functions |
    # +----------------------------+
    
    def discoverDevices(self):
        '''Find and create devices'''
        
        self.cores = {}
        self.createModelCore("cpu0", "ARM_Cortex-M55", "Cortex-M55")
        
    def createModelCore(self, deviceName, dtslName, coreDefinition):
        try:
            self.cores[deviceName] = ConnectableDevice(self, self.findDevice(deviceName), dtslName)
            deviceInfo = DeviceInfo("core", coreDefinition)
            self.cores[deviceName].setDeviceInfo(deviceInfo)
        except:
            # Core failed to be added
            pass
    
    def exposeCores(self):
        '''Expose cores'''
        for device in self.cores.values():
            self.addDeviceInterface(device)
    
    def getTraceCaptureForSource(self, traceSource):
        if traceSource in self.traceSources.values():
            return self.traceCaptures.values()[self.traceSources.values().index(traceSource)]
        return None
    
    def getManagedDevices(self):
        '''Get the required list of managed devices for this configuration'''
        deviceList = self.mgdPlatformDevs[:]
        deviceList.extend(self.traceSources.values())
        deviceList.extend(self.traceCaptures.values())
        return deviceList
    
    # +--------------------------------+
    # | Callback functions for options |
    # +--------------------------------+
    
    def optionValuesChanged(self):
        '''Callback to update the configuration state after options are changed'''
        if not self.isConnected():
            self.setInitialOptions()
        
        self.updateDynamicOptions()
        
    def setInitialOptions(self):
        '''Set the initial options'''
        
        traceMode = self.getOptionValue("options.trace.traceCapture")
        traceBufferSize = self.getOptionValue("options.trace.traceCapture.modelCapture.traceBufferSize")
        streamID = 1
        sinkNum = 0
        self.traceSources = {}
        self.traceCaptures = {}
        for i in range(len(self.cores)):
            enableTraceForCore = self.getOptionValue("options.coreTrace.core%d" % i) and traceMode == "modelStreamingCapture"
            if enableTraceForCore:
                self.traceSources[i] = ModelInstTraceSource(self, self.findDevice(traceDeviceNames[i]), streamID, "model_instr_trace_%d" % i)
                self.traceSources[i].setEnabled(True)
                streamingCapture = ModelStreamingTraceCapture(self, "model_streaming_%d" % i, "model_trace_sink_%d" % sinkNum)
                self.traceCaptures[i] = ModelLiveStoredStreamingTraceCapture(self, streamingCapture, "model_streaming_capture_%d" % i)
                self.addTraceCaptureInterface(self.traceCaptures[i])
                self.addStreamTraceCaptureInterface(self.traceCaptures[i])
                self.traceCaptures[i].addTraceSource(self.traceSources[i], coreNames[i])
                self.setModelTraceBufferSize(traceBufferSize, self.traceCaptures[i])
                sinkNum += 1
            streamID += 1
        
        self.setManagedDeviceList(self.getManagedDevices())
        
    def updateDynamicOptions(self):
        '''Update the dynamic options'''
        
    def setModelTraceBufferSize(self, traceBufferSize, traceCapture):
        '''Configuration option setter method for the buffer size'''
        captureSize = 64*1024*1024
        if (traceBufferSize == "64MB"):
            captureSize = 64*1024*1024
        elif (traceBufferSize == "128MB"):
            captureSize = 128*1024*1024
        elif (traceBufferSize == "256MB"):
            captureSize = 256*1024*1024
        elif (traceBufferSize == "512MB"):
            captureSize = 512*1024*1024
        elif (traceBufferSize == "1GB"):
            captureSize = 1*1024*1024*1024
        elif (traceBufferSize == "2GB"):
            captureSize = 2*1024*1024*1024
        elif (traceBufferSize == "4GB"):
            captureSize = 4*1024*1024*1024
        elif (traceBufferSize == "8GB"):
            captureSize = 8*1024*1024*1024
        elif (traceBufferSize == "16GB"):
            captureSize = 16*1024*1024*1024
        elif (traceBufferSize == "32GB"):
            captureSize = 32*1024*1024*1024
        elif (traceBufferSize == "64GB"):
            captureSize = 64*1024*1024*1024
        elif (traceBufferSize == "128GB"):
            captureSize = 128*1024*1024*1024
        else:
            raise ValueError("Unexpected trace buffer size: ", traceBufferSize)
        
        traceCapture.setMaxCaptureSize(captureSize)
    
    # +------------------------------+
    # | Target independent functions |
    # +------------------------------+
    
    @staticmethod
    def getOptionTraceBufferTabPage():
        return DTSLv1.tabPage("trace", "Trace Buffer", childOptions=[
                DTSLv1.radioEnumOption("traceCapture", "Trace capture method", defaultValue="none",
                    values = [("none", "None"), DtslScript.getModelTraceOptions()]),
        ])
    
    @staticmethod
    def getModelTraceOptions():
        return ("modelStreamingCapture", "Model Trace",
                DTSLv1.infoElement("modelCapture", "", "", childOptions=[
                    DTSLv1.enumOption("traceBufferSize", "Trace Buffer Size", defaultValue="4GB",
                        values = [("64MB", "64MB"), ("128MB", "128MB"), ("256MB", "256MB"),
                                  ("512MB", "512MB"), ("1GB", "1GB"), ("2GB", "2GB"),
                                  ("4GB", "4GB"), ("8GB", "8GB"), ("16GB", "16GB"),
                                  ("32GB", "32GB"), ("64GB", "64GB"), ("128GB", "128GB")], isDynamic=False)
                ])
        )
    
    @staticmethod
    def getCoreTraceTabPage():
        return DTSLv1.tabPage("coreTrace", "Core Trace", childOptions=[
            DTSLv1.booleanOption("core0", "Enable ARM_Cortex-M55 trace", defaultValue=False),
        ])
    
